#!/usr/bin/perl -w
##
# Caching URL fetcher for my scripts.
#
# Not intended to be clever, just to give us some simple way
# to keep a cache of the remote content so we don't hit the
# network so often.
#

package Gerph::CachedFetcher;

use Digest::MD5;
use LWP::Simple qw($ua);
use LWP::Simple::Cookies;

my $fetchcache = '/tmp/';
$fetchcache = '/var/jfcache/'; # Local variant, not in /tmp

# Should be overridden at creation time.
my $category = 'unknown';

# How long to cache by default
our $defaultlifetime = 60*60*4;

# Our agent name to use
our $agentname = "GerphCachedFetcher/0.2";

# Debugging this fetcher?
our $debug = 0;

# Statistics for use
our %stats = (
    'fetched' => 0,
    'cached'  => 0,
    'fetched_size' => 0,
    'cached_size' => 0,
);

END {
    # Only print the statistics if we did anything.
    if ($debug)
    {
        if ($stats{'fetched'} || $stats{'cached'})
        {
            printf "Fetched URLs: %d\n", $stats{'fetched'};
            printf "Fetched data: %.2f MB\n", $stats{'fetched_size'}/1024/1024;
            printf "Cached URLs:  %d\n", $stats{'cached'};
            printf "Cached data:  %.2f MB\n", $stats{'cached_size'}/1024/1024;
        }
    }
}

##
# Create a new fetcher for a given site
#
# @param[in] $class     Class we are an instance of
# @param[in] $category  The category we're caching for (top level directory),
#                       eg 'amazon', 'news', 'ratings'
# @param[in] $site      The site we're a fetcher for
# @param[in] $lifetime  Default lifetime of a fetch
#
# @return new instance
sub new
{
    my ($class, $category, $site, $lifetime) = @_;
    
    # If not supplied, use 'root' as the name
    $site //= 'root';
    
    # Make the site safe to use in a name
    $site =~ s/[^A-Za-z0-9-_]/_/g;
    my $self = {
        'category'  => $category,
        'cachedir'  => "$fetchcache/$category",
        'site'      => $site,
        'lifetime'  => $lifetime || $defaultlifetime
    };

    bless $self, $class;

    return $self;
}


##
# Fetch a URL, caching if necessary
#
# @param[in] $self     Our cache object
# @param[in] $url      URL to fetch
# @param[in] $lifetime Cache duration in seconds
#
# @return contents of the url/cache, or undef if cannot fetch
sub fetch
{
    my ($self, $url, $lifetime) = @_;
    
    # Default cache lifetime of 15 minutes
    $lifetime ||= $self->{'lifetime'};
    
    my $url_md5 = Digest::MD5::md5_hex($url);
    my $cache = "$self->{'cachedir'}/$self->{'site'}/$url_md5";

    #print "Fetch: $url_md5 $url lifetime $lifetime\n";
    
    # Create directories if not present
    mkdir "$fetchcache" if (! -d "$fetchcache");
    mkdir "$self->{'cachedir'}" if (! -d "$self->{'cachedir'}");
    mkdir "$self->{'cachedir'}/$self->{'site'}" if (! -d "$self->{'cachedir'}/$self->{'site'}");
    
    my $data;
    if (-f $cache)
    {
        my @st = stat($cache);
        my $mtime = $st[9];
        if (time() - $mtime < $lifetime)
        {
            # The file exists in the cache, so we can use it.
            if (open(my $fh, "<", $cache))
            {
                sysread($fh, $data, -s $cache);
                close($fh);
                $stats{'cached'}++;
                $stats{'cached_size'} += length $data;
            }
        }
    }
    
    if (!defined $data)
    {
        print "Fetching data from server for $url\n" if ($debug);
        $LWP::Simple::ua->agent($agentname);
        $data = LWP::Simple::get($url);
        
        if (defined $data &&
            open(my $fh, ">", $cache))
        {
            print "  got data\n" if ($debug);
            print $fh $data;
            close($fh);
        }
        else
        {
            print "  got no data (or could not write to $cache)?\n" if ($debug);
        }
        $stats{'fetched'}++;
        if (defined $data)
        {
            $stats{'fetched_size'} += length $data;
        }
    }
    
    return $data;
}

1;
