#!/usr/bin/perl
##
# Top Films fetcher - provide details about the top films at different
# sites as a set of XML files.
#
# The XML files have the following elements:
#
# <chart>
#   The root chart element, holding all the chart information.
#       @title  The chart title, as a readable string
#       @site   The site name, as a readable string
#
# <film>
#   The films present in this chart, in numerical order.
#       @number Chart position, as an integer number. May duplicate other
#               numbers in the chart, if films are at joint positions.
#       @title  The film title.
#       @year   The year of the film's release (optional).
#       @url    A related URL for the film (optional).
#       @img    A related image for the film (optional).
#
# Results are fetched to the directory 'filmcharts'.
#
# Fetch everything to XML files:
#   ./TopFilms-fetcher.pl
#
# For testing purposes:
#   ./TopFilms-fetcher.pl -site <sitename>
#       Fetch the XML files for a given site.
#   ./TopFilms-fetcher.pl -site <sitename> -chart <chartname>
#       Fetch a chart from the site and display it (as a perl Dumper output).
#

# Common modules.
use XML::Simple;
use File::Slurp;
use Data::Dumper;
use Getopt::Long;

# Local modules.
use TopFilms;


my $outputdir = "filmcharts";
my $sites = TopFilms::enumerate();
my $sitematch;
my $chartmatch;

GetOptions("site=s" => \$sitematch,
           "chart=s" => \$chartmatch)
    || exit 1;

if (defined $chartmatch)
{
    die "-site must be given if you want to use -chart" if (!defined $sitematch);
    # And the site must completely match.
    my $site = TopFilms::site($sitematch);
    die "No site '$sitematch'" if (!defined $sitematch);
    
    my $chart = $site->chart($chartmatch);
    die "No chart '$chartmatch'" if (!defined $chartmatch);
    
    print Dumper($chart);
    exit;
}

# Unbuffered output.
$| = 1;

mkdir $outputdir;
for my $sitename (@$sites)
{
    if (defined $sitematch &&
        $sitename !~ /$sitematch/)
    {
        next;
    }
    my $site = TopFilms::site($sitename);
    
    next if (!defined $site);
    
    my $sitetitle = $site->name();
    print "Process site: $sitename ($sitetitle)\n";
    
    # For every chart, we're going to write out a file (if we could)
    my $charts = $site->charts();
    for my $chartname (keys %$charts)
    {
        print "  Fetching chart '$chartname': ";
        my $chart = $site->chart($chartname);
        if (!defined $chart)
        {
            print "No chart returned\n";
            print STDERR "Cannot fetch from site '$sitename' chart '$chartname'\n";
            next;
        }
        
        if (scalar(@$chart) == 0)
        {
            print "No data\n";
            print STDERR "No data from site '$sitename' chart '$chartname'\n";
            next;
        }
        
        print "OK\n";
        my $xml = XML::Simple::XMLout( { 'site' => $sitetitle,
                                         'title' => $charts->{$chartname}->{'name'},
                                         'film' => $chart },
                                         KeepRoot => 1,
                                       RootName => 'chart' );
        write_file( "$outputdir/$sitename-$chartname.xml", 
                    { binmode => ':utf8' },
                    $xml );
    }
}
