#!/usr/bin/perl
## 
# TopFilms: Module for handling charts of the best films in categories,
# from different sites.
#
# Expected use:
#
#    # List all the sites available
#    my $sitelist = TopFilms::enumerate();
#
#    # Get the configuration for a given site.
#    my $site = TopFilms::site($sitename);
#    my $sitetitle = $site->name();
#
#    # A list of the charts that we can generate through this site, as a
#    # hashref, keyed by the chart name:
#    my $charts = $site->charts();
#
#    my $charttitle = $chart->{$chartname}->{'title'};
#    my $chart = $site->chart($chartname);
#
# We cache the fetches through the Gerph::CacheFetcher module, which keeps
# the fetched pages so that we don't have to go to the site so often.
#

package TopFilms;

use Data::Dumper;
use Gerph::CachedFetcher;
use Encode;

# How long to cache by default (14 days?)
my $default_lifetime = 60*60*24*14;


##
# Enumerate the sites.
#
# @return arrayref of site names
sub enumerate
{
    my (@sites);
    opendir(my $dirfh, "TopFilms") || return [];
    while (my $file = readdir($dirfh))
    {
        if ($file =~ /^(.*)\.pm$/)
        {
            push @sites, $1;
        }
    }
    closedir($dirfh);

    return \@sites;
}


##
# Create a new site processor
#
# @param[in] $sitename Site name to include
#
# @return instantiated site processor, or undef if could not create
sub site
{
    my ($sitename) = @_;

    die "Cannot instantiate a site without a sitename" if (!defined $sitename);

    my $site = eval "
                    require 'TopFilms/$sitename.pm';
                    new TopFilms::$sitename;
               ";
    if ($@)
    {
        print STDERR "Failed to create TopFilms::$sitename - $@\n";
        return undef;
    }

    return $site;
}


##
# Create a new TopFilms object (from which all the fetches inherit)
#
# @param[in] $class  Class name we're creating
#
# @return blessed object
sub new
{
    my ($class) = @_;
    if ($class eq 'TopFilms')
    {
        die "Don't create TopFilms this way\n";
    }
    my $site = $class;
    $site =~ s/^TopFilms:://;

    my $self = {
        'site'    => $site,

        # Default lifetime on articles
        'fetcher' => new Gerph::CachedFetcher('topfilms', $site, $default_lifetime),
    };

    bless $self, $class;

    return $self;
}



##
# Fetch a URL through the cache.
#
# @param[in] $self      This object
# @param[in] $url       URL to fetch
# @param[in] $lifetime  cache lifetime
#
# @return the fetched data, or undef if cannot parse
sub fetch
{
    my ($self, $url, $lifetime) = @_;

    my $data = $self->{'fetcher'}->fetch($url, $lifetime);

    return $data;
}


##
# Get a list of the charts that this fetcher supports.
#
# @param[in] $self  This object
#
# @return hashref keyed by the chart name, each value a hashref containing:
#           'name'  => The name of the chart for presentation
#           'url'   => The URL that will be fetched.
#           
sub charts
{
    my ($self) = @_;
    if (!defined $self->{'charts'})
    {
        my $charts = $self->getCharts();
        $self->{'charts'} = $charts;
    }
    return $self->{'charts'};
}

##
# Fetch a specific chart, by name.
#
# @param[in] $self      This object
# @param[in] $name      The name of the chart as a keyname from the charts
#
# @retval undef if not found
# @return arrayref containing the charts read, each element of which contains
#         a hashref keyed:
#           'number'    charting number
#           'title'     film title
#           'year'      year (optionally)
#           'url'       some url (optionally)
#           'img'       some image url (optionally)
#         The arrayref will be ordered upwards from 1.
sub chart
{
    my ($self, $name) = @_;

    # Get the chart details
    my $chartref = $self->charts()->{$name};
    
    if (!defined $chartref)
    {
        # Could not fetch; failed.
        return undef;
    }
    if (!defined $chartref->{'url'})
    {
        return undef;
    }

    # The chart we get our results into.
    my $totalchart = [];
    
    # For each URL in the list (or just the one if they old supplied one,
    # process the pages).
    my @urls = ref($chartref->{'url'}) eq 'ARRAY' ? @{ $chartref->{'url'} } : ( $chartref->{'url'} );

    my $fetchedok = 0;    
    for my $url (@urls)
    {    
        # Call the fetcher to get the cached results.
        print "Fetching $url\n" if ($chartref->{'debug'});
        my $page = $self->{'fetcher'}->fetch( $url );
        if (!defined $page)
        {
            print "  No page was fetched\n" if ($chartref->{'debug'});
            next;
        }
        $fetchedok = 1;

        # The chart we get our results into.
        my $chart = [];

        # Need to call the named function - we might have a pattern function to
        # make their lives easier.
        if ($self->can( "pattern$name" ) || 
            $self->can( "pattern" ))
        {
            # We have a pattern function, so we can do all the work for them.
            my $func = $self->can( "pattern$name" ) ? "pattern$name" : "pattern";
            my $pattern = $self->$func();

            my @entries = ();
            if (defined $pattern->{'entry'})
            {
                @entries = ($page =~ /$pattern->{'entry'}/g);
            }
            elsif (defined $pattern->{'split'})
            {
                @entries = split /$pattern->{'split'}/, $page;
            }
            else
            {
                die "Do not know how to generate entries for pattern '$name'";
            }
            if ($chartref->{'debug'})
            {
                print "Read ", scalar(@entries), " entries from page\n";
            }
            for my $li (@entries)
            {
                my ($number, $title, $year, $url, $img);
                ($number) = ($li =~ /$pattern->{'number'}/) if (defined $pattern->{'number'});
                ($title) = ($li =~ /$pattern->{'title'}/);
                ($year) = ($li =~ /$pattern->{'year'}/) if (defined $pattern->{'year'});
                ($img) = ($li =~ /$pattern->{'img'}/) if (defined $pattern->{'img'});
                ($url) = ($li =~ /$pattern->{'url'}/) if (defined $pattern->{'url'});

                if (defined $title)
                {
                    my $entry = {
                            'number' => $number,
                            'title'  => $title,
                            defined $year ? ('year' => $year) : (),
                            defined $url ? ('url' => $url) : (),
                            defined $img ? ('img' => $img) : (),
                        };
                    # If they supplied a cleaning function, call it.
                    if (defined $pattern->{'clean'})
                    {
                        $entry = &{ $pattern->{'clean'} } ( $entry );
                    }
                    push @$chart, $entry;
                }
            }

        }
        else
        {
            my $func = $self->can( "chart$name" ) ? "chart$name" : "chart";
            $chart = $self->$func( $page );
        }
        
        # Append our new chart data to the existing chart data
        push @$totalchart, @$chart;
    }
    
    # Check if we were actually able to get anything from the site
    if (!$fetchedok)
    {
        return undef;
    }

    # Now tidy up the charts by providing an incrementing number
    # where one wasn't present
    my $position = 0;
    for my $item (@$totalchart)
    {
        if (!defined $item->{'number'})
        {
            $item->{'number'} = ++$position;
        }
        else
        {
            $position = item->{'number'};
        }
    }
    
    # We may need to decode the names.
    if (defined $chartref->{'charset'})
    {
        for my $item (@$totalchart)
        {
            $item->{'title'} = Encode::decode($chartref->{'charset'}, $item->{'title'});
        }
    }
    
    # Sort the list, just so that it's tidier.
    $totalchart = [ sort { $a->{'number'} <=> $b->{'number'} } @$totalchart ];
    
    return $totalchart;
}


# Must return true.
1;
